@@ -1,4 +1,4 @@
-#!/usr/local/bin/php-5.4
+#!/usr/local/bin/php-5.5
 <?php
 /*
  +-----------------------------------------------------------------------+
@@ -52,7 +52,7 @@
  *  - you are dealing with counterfeit header data.
  */
 
-define('INSTALL_PATH', realpath(dirname(__FILE__).'/..') . '/');
+define('INSTALL_PATH', realpath(__DIR__ .'/..') . '/');
 
 require INSTALL_PATH . 'program/include/clisetup.php';
 
@@ -60,7 +60,7 @@ if ($argc < 2) {
 	die("Usage: " . basename($argv[0]) . " encrypted-hdr-part [encrypted-hdr-part ...]\n");
 }
 
-$RCMAIL = rcmail::get_instance();
+$RCMAIL = rcube::get_instance();
 
 for ($i = 1; $i < $argc; $i++) {
 	printf("%s\n", $RCMAIL->decrypt($argv[$i]));
