@@ -1,6 +1,6 @@
 <?php
 
-if (!class_exists('rcube_install') || !is_object($RCI)) {
+if (!class_exists('rcmail_install', false) || !is_object($RCI)) {
     die("Not allowed! Please open installer/index.php instead.");
 }
 
@@ -202,9 +202,6 @@ echo '<label for="cfgdebug1">Log errors</label><br />';
 echo $check_debug->show(($value & 4) ? 4 : 0, array('value' => 4, 'id' => 'cfgdebug4'));
 echo '<label for="cfgdebug4">Print errors (to the browser)</label><br />';
 
-echo $check_debug->show(($value & 8) ? 8 : 0, array('value' => 8, 'id' => 'cfgdebug8'));
-echo '<label for="cfgdebug8">Verbose display (enables debug console)</label><br />';
-
 ?>
 </dd>
 
@@ -674,6 +671,28 @@ echo $select_param_folding->show(strval($RCI->getprop('mime_param_folding')));
 <p class="hint"><span class="userconf">*</span>&nbsp; These settings are defaults for the user preferences</p>
 </fieldset>
 
+
+<fieldset>
+<legend>Plugins</legend>
+<dl class="configblock" id="cgfblockdisplay">
+
+<?php
+$plugins = $RCI->list_plugins();
+foreach($plugins as $p) 
+{
+    $p_check = new html_checkbox(array('name' => '_plugins_'.$p['name'], 'id' => 'cfgplugin_'.$p['name'], 'value' => $p['name']));
+    echo '<dt class="propname"><label>';
+    echo $p_check->show($p['enabled'] ? $p['name'] : 0);
+    echo '&nbsp;' . $p['name'] . '</label></dt><dd>';
+    echo '<label for="cfgplugin_'.$p['name'].'" class="hint">' . $p['desc'] . '</label><br/></dd>';
+}
+
+?>
+</dl>
+
+<p class="hint">Please consider checking dependencies of enabled plugins</p>
+</fieldset>
+
 <?php
 
 echo '<p><input type="submit" name="submit" value="' . ($RCI->configured ? 'UPDATE' : 'CREATE') . ' CONFIG" ' . ($RCI->failures ? 'disabled' : '') . ' /></p>';
