@@ -3,9 +3,9 @@
 /*
  +-------------------------------------------------------------------------+
  | Roundcube Webmail setup tool                                            |
- | Version 0.8                                                             |
+ | Version 1.1.0                                                           |
  |                                                                         |
- | Copyright (C) 2009-2012, The Roundcube Dev Team                         |
+ | Copyright (C) 2009-2015, The Roundcube Dev Team                         |
  |                                                                         |
  | This program is free software: you can redistribute it and/or modify    |
  | it under the terms of the GNU General Public License (with exceptions   |
@@ -34,39 +34,42 @@
  +-------------------------------------------------------------------------+
  | Author: Thomas Bruederli <roundcube@gmail.com>                          |
  +-------------------------------------------------------------------------+
-
- $Id$
-
 */
 
-ini_set('error_reporting', E_ALL&~E_NOTICE);
+ini_set('error_reporting', E_ALL &~ (E_NOTICE | E_STRICT));
 ini_set('display_errors', 1);
 
-define('INSTALL_PATH', realpath(dirname(__FILE__) . '/../').'/');
-define('RCMAIL_CONFIG_DIR', INSTALL_PATH . 'config');
-define('RCMAIL_CHARSET', 'UTF-8');
+define('INSTALL_PATH', realpath(__DIR__ . '/../').'/');
+define('RCUBE_INSTALL_PATH', INSTALL_PATH);
+define('RCUBE_CONFIG_DIR', INSTALL_PATH . 'config/');
 
 $include_path  = INSTALL_PATH . 'program/lib' . PATH_SEPARATOR;
-$include_path .= INSTALL_PATH . 'program' . PATH_SEPARATOR;
 $include_path .= INSTALL_PATH . 'program/include' . PATH_SEPARATOR;
 $include_path .= ini_get('include_path');
 
 set_include_path($include_path);
 
-require_once 'utils.php';
-require_once 'main.inc';
+// include composer autoloader (if available)
+if (@file_exists(INSTALL_PATH . 'vendor/autoload.php')) {
+    require INSTALL_PATH . 'vendor/autoload.php';
+}
 
-session_start();
+require_once 'Roundcube/bootstrap.php';
+// deprecated aliases (to be removed)
+require_once 'bc.php';
 
-$RCI = rcube_install::get_instance();
+if (function_exists('session_start'))
+  session_start();
+
+$RCI = rcmail_install::get_instance();
 $RCI->load_config();
 
-if (isset($_GET['_getfile']) && in_array($_GET['_getfile'], array('main', 'db'))) {
-  $filename = $_GET['_getfile'] . '.inc.php';
-  if (!empty($_SESSION[$filename])) {
+if (isset($_GET['_getconfig'])) {
+  $filename = 'config.inc.php';
+  if (!empty($_SESSION['config'])) {
     header('Content-type: text/plain');
     header('Content-Disposition: attachment; filename="'.$filename.'"');
-    echo $_SESSION[$filename];
+    echo $_SESSION['config'];
     exit;
   }
   else {
@@ -76,14 +79,14 @@ if (isset($_GET['_getfile']) && in_array($_GET['_getfile'], array('main', 'db'))
 }
 
 if ($RCI->configured && ($RCI->getprop('enable_installer') || $_SESSION['allowinstaller']) &&
-    isset($_GET['_mergeconfig']) && in_array($_GET['_mergeconfig'], array('main', 'db'))) {
-  $filename = $_GET['_mergeconfig'] . '.inc.php';
+    !empty($_GET['_mergeconfig'])) {
+  $filename = 'config.inc.php';
 
   header('Content-type: text/plain');
   header('Content-Disposition: attachment; filename="'.$filename.'"');
 
   $RCI->merge_config();
-  echo $RCI->create_config($_GET['_mergeconfig'], true);
+  echo $RCI->create_config();
   exit;
 }
 
@@ -123,8 +126,16 @@ if ($RCI->configured && empty($_REQUEST['_step'])) {
   // exit if installation is complete
   if ($RCI->configured && !$RCI->getprop('enable_installer') && !$_SESSION['allowinstaller']) {
     // header("HTTP/1.0 404 Not Found");
-    echo '<h2 class="error">The installer is disabled!</h2>';
-    echo '<p>To enable it again, set <tt>$rcmail_config[\'enable_installer\'] = true;</tt> in RCMAIL_CONFIG_DIR/main.inc.php</p>';
+    if ($RCI->configured && $RCI->legacy_config) {
+      echo '<h2 class="error">Your configuration needs to be migrated!</h2>';
+      echo '<p>We changed the configuration files structure and your installation needs to be updated accordingly.</p>';
+      echo '<p>Please run the <tt>bin/update.sh</tt> script from the command line or set <p>&nbsp; <tt>$rcube_config[\'enable_installer\'] = true;</tt></p>';
+      echo ' in your RCUBE_CONFIG_DIR/main.inc.php to let the installer help you migrating it.</p>';
+    }
+    else {
+      echo '<h2 class="error">The installer is disabled!</h2>';
+      echo '<p>To enable it again, set <tt>$config[\'enable_installer\'] = true;</tt> in RCUBE_CONFIG_DIR/config.inc.php</p>';
+    }
     echo '</div></body></html>';
     exit;
   }
