@@ -5,7 +5,7 @@
  | program/include/iniset.php                                            |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2013, The Roundcube Dev Team                       |
+ | Copyright (C) 2008-2015, The Roundcube Dev Team                       |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -21,7 +21,7 @@
 */
 
 // application constants
-define('RCMAIL_VERSION', '0.9.2');
+define('RCMAIL_VERSION', '1.1.0');
 define('RCMAIL_START', microtime(true));
 
 if (!defined('INSTALL_PATH')) {
@@ -54,6 +54,11 @@ if (set_include_path($include_path) === false) {
 // (does not work in safe mode)
 @set_time_limit(120);
 
+// include composer autoloader (if available)
+if (@file_exists(INSTALL_PATH . 'vendor/autoload.php')) {
+    require INSTALL_PATH . 'vendor/autoload.php';
+}
+
 // include Roundcube Framework
 require_once 'Roundcube/bootstrap.php';
 
@@ -63,6 +68,11 @@ spl_autoload_register('rcmail_autoload');
 // backward compatybility (to be removed)
 require_once INSTALL_PATH . 'program/include/bc.php';
 
+// load the UTF-8 portablity layer from Patchwork
+if (!function_exists('iconv') || !function_exists('utf8_encode') || !extension_loaded('mbstring')) {
+    \Patchwork\Utf8\Bootup::initAll();
+}
+
 
 /**
  * PHP5 autoloader routine for dynamic class loading
@@ -79,4 +89,3 @@ function rcmail_autoload($classname)
 
     return false;
 }
-
